/***************************************************************************//**
 *   @file   ADXRS453.c
 *   @brief  Implementation of ADXRS453 Driver.
 *   @author Bancisor MIhai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 408
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "ADXRS453.h"			// ADXRS453 definitions.
#include "Communication.h"		// Communication definitions.

/***************************************************************************//**
 * @brief Sets or clears the parity bit in order to ensure that the overall parity 
 * 		  of the data word is odd.
 *
 * @param data.
 *
 * @return parityBit.
*******************************************************************************/
unsigned char ParityBit(unsigned long data)
{
	unsigned char parityBit = 0;
	unsigned char bit = 0;
	unsigned char sum = 0;
	
	for(bit = 0; bit < 32; bit++)
	{
		sum += ((data >> bit) & 0x1);	
	}
	if (! (sum % 2))
	{
		parityBit |= 0x1;
	}	
		
	return parityBit;	
}
 

/***************************************************************************//**
 * @brief Initializes the ADXRS453 and checks if the device is present.
 *
 * @return status - Result of the initialization procedure.
 *                  Example: 1 - if initialization was successful (ID starts
 *                               with 0x52).
 *                           0 - if initialization was unsuccessful.
*******************************************************************************/
unsigned char ADXRS453_Init(void)
{ 
	unsigned char status = 0x1;
	unsigned char data[5] = {0, 0, 0, 0, 0};
	
	data[4] = 3;
	data[3] = 0;
    data[2] = 0;
	data[1] = 2;
	data[0] = 1;
	SPI_Write(data,4);
	DelaySeconds(0.05);	
	data[4] = 0;
	data[3] = 0;
    data[2] = 0;
	data[1] = 2;
	data[0] = 1;
	SPI_Write(data,4);
	DelaySeconds(0.05);		
	data[4] = 0;
	data[3] = 0;
    data[2] = 0;
	data[1] = 2;
	data[0] = 1;
	SPI_Write(data,4);
	DelaySeconds(0.05);		
	data[4] = 0;
	data[3] = 0;
    data[2] = 0;
	data[1] = 2;
	data[0] = 1;
	SPI_Write(data,4);
	DelaySeconds(0.1);
	if((ADXRS456_GetRegisterValue(ADXRS453_PID1) >> 8) != 0x52)
	{
		status = 0x0;
	}
	
	return status;
}

/***************************************************************************//**
 * @brief Reads data from ADXRS453.
 *
 * @return receivedData.
*******************************************************************************/
unsigned long ADXRS456_Data(void)
{
	    unsigned char data[5] = {0x00, 0x00, 0x00, 0x00, 0x00};
		unsigned long receivedData = 0x00;
		
		data[0] = 1;
		data[1] = ADXRS453_SENSOR_DATA;
		SPI_Read(data,4);
		receivedData += (data[0] << 24);
		receivedData += (data[1] << 16);
		receivedData += (data[2] << 8);
		receivedData = ((receivedData >> 10) & 0xFFFF);
		
		return receivedData;
}
/***************************************************************************//**
 * @brief Reads register from ADXRS453.
 *
 * @param regAddress - the address of the register.
 *
 * @return receivedData.
*******************************************************************************/
unsigned short ADXRS456_GetRegisterValue(unsigned char regAddress)
{
	    unsigned char data[5] = {0x00, 0x00, 0x00, 0x00, 0x00};
		unsigned long receivedData = 0x00;	
		unsigned long dataToSend = 0;
		
		data[0] = 1;
		data[1] = ADXRS453_READ_DATA | (regAddress >> 7);
		data[2] = regAddress << 1;
		dataToSend += (data[1] << 24);
		dataToSend += (data[2] << 16);
		data[4] = ParityBit(dataToSend);
		SPI_Write(data,4);
		SPI_Read(data,4);	// because the device answers only on the next SPI word
		receivedData += (data[1] << 16);
		receivedData += (data[2] << 8);
		receivedData += (data[3] << 0);
		receivedData = ((receivedData >> 5) & 0xFFFF);
		
		return (unsigned short)receivedData;
}
/***************************************************************************//**
 * @brief Writes register to ADXRS453.
 *
 * @param regAddress - the address of the register.
 *
 * @param regData - data to write to the register 
 *
 * @return receivedData.
*******************************************************************************/
void ADXRS456_SetRegisterValue(unsigned char regAddress, unsigned short regData)
{
	    unsigned char data[5] = {0x00, 0x00, 0x00, 0x00, 0x00};
		unsigned long dataToSend = 0;
		
		data[0] = 1;
		data[1] = ADXRS453_READ_DATA | (regAddress >> 7);
		data[2] = (regAddress << 1) | (regData >> 15);
		data[3] = (regData & 0x7F80) >> 7;
		data[4] = (regData & 0xFF) << 1;
		dataToSend += (data[1] << 24);
		dataToSend += (data[2] << 16);
		data[4] = ParityBit(dataToSend);
		SPI_Write(data,4);
}
/***************************************************************************//**
 * @brief Reads temperature from ADXRS453 and converts it to degrees Celsius
 *
 * @param None.
 *
 * @return temperature.
*******************************************************************************/
unsigned char ADXRS456_GetTemperature(void)
{
	unsigned short temp = 0;
	
	temp = ADXRS456_GetRegisterValue(ADXRS453_TEMP1);
	temp >>= 6;	
	temp -= 0x31f;
	temp /= 5;
	
	return (unsigned char)temp;
}